/* 
 * Temperature Sensor Transmitter
 * 
 * Remote temperature sensor transmitter using a 2400 bps
 * 433.92 MHz ASK transmitter and a TMP36 temperature sensor.
 *
 * File:   temperature_transmitter.c
 * Author: Ryan Clarke
 *
 * Pin assignments:
 *     RA0 - Switching transistor & external devices (OUTPUT)
 *     RA1 - LM4040 2.048V reference
 *     RA2 - TMP36 analog input (AN2)
 *     RA3 - MCLR
 *     RA4 - EUSART TX (OUTPUT)
 *     RA5 - EUSART RX
 *
 */

#include <xc.h>


/*
 * Device configuration words:
 *     Internal/external switch-over - disabled (two-speed startup disabled)
 *     Oscillator - internal
 *     Fail-safe clock monitor - disabled (not required, no external oscillator)
 *     PLL - disabled
 *     Brown out reset threshold - high (2.7 V typical)
 *     Low voltage programming - disabled
 *
 * All others left at their defaults.
 */
#pragma config IESO = OFF
#pragma config FOSC = INTOSC
#pragma config FCMEN = OFF
#pragma config PLLEN = OFF
#pragma config BORV = HI
#pragma config LVP = OFF

/*
 * Macros
 */
#define _XTAL_FREQ 500000

#define DEVICE_ID  0x11;
#define PREAMBLE   0b10101010;


/*
 * Main function
 */
void main(void)
{
    unsigned char checksum;

    // 64 second WDT interval
    WDTCONbits.WDTPS = 0b10000;

    // Clear and configure PORTA
    PORTA = 0;
    TRISA = 0b101110;
    
    // Enable analog input on RA2/AN2
    ANSELA = 0b00100;

    // EUSART configuration:
    // RX/TX on RA5/RA4
    // 2400 8N1 asynchronous
    // 2400 bps desired, 2403.85 calculated
    // 0.16% bit error rate
    APFCONbits.RXDTSEL = 1;
    APFCONbits.TXCKSEL = 1;
    SPBRGL = 12;
    SPBRGH = 0;
    TXSTAbits.BRGH = 1;
    TXSTAbits.SYNC = 0;
    RCSTAbits.SPEN = 1;

    // ADC configuration:
    // Fosc/2 conversion clock
    // Vref+ connected to external Vref+
    // AN2 analog channel
    // Right justified
    ADCON1bits.ADCS = 0b000;
    ADCON1bits.ADPREF = 0b10;
    ADCON0bits.CHS = 0b00010;
    ADCON1bits.ADFM = 1;

    while(1)
    {
        // Power-up the transmitter, LM4040, and TMP36
        PORTAbits.RA0 = 1;

        // Delay for the TMP36 and LM4040 to stabilize
        __delay_ms(1);

        // Enable the ADC and EUSART TX
        ADCON0bits.ADON = 1;
        TXSTAbits.TXEN = 1;

        // Wait for Tacq, start a conversion, and wait until it is done
        __delay_us(5);
        ADCON0bits.ADGO = 1;
        while(ADCON0bits.ADGO);

        // Compute the checksum using a modular sum
        checksum = DEVICE_ID + ADRESH + ADRESL;
        checksum = (~checksum) + 1;

        // Transmit a preamble, device ID, and data
        while(!TXSTAbits.TRMT);
        TXREG = PREAMBLE;
        while(!TXSTAbits.TRMT);
        TXREG = PREAMBLE;
        while(!TXSTAbits.TRMT);
        TXREG = DEVICE_ID;
        while(!TXSTAbits.TRMT);
        TXREG = ADRESH;
        while(!TXSTAbits.TRMT);
        TXREG = ADRESL;
        while(!TXSTAbits.TRMT);
        TXREG = checksum;
        while(!TXSTAbits.TRMT);

        // Disable the EUSART TX and ADC
        TXSTAbits.TXEN = 0;
        ADCON0bits.ADON = 0;

        // Power-down the transmitter, LM4040, and TMP36
        PORTAbits.RA0 = 0;

        SLEEP();
    }
}
